import { type ValidationContext } from '../../validation/validation-context';
import { type BinaryExpression, type TernaryExpression, type UnaryExpression } from '../generated/ast';
import { type WrapperFactoryProvider } from '../wrappers';
import { type EvaluationContext } from './evaluation-context';
import { EvaluationStrategy } from './evaluation-strategy';
import { type InternalValueRepresentation, type InternalValueRepresentationTypeguard } from './internal-value-representation';
import { type BinaryExpressionOperator, type TernaryExpressionOperator, type UnaryExpressionOperator } from './operator-types';
export interface OperatorEvaluator<E extends UnaryExpression | BinaryExpression | TernaryExpression> {
    readonly operator: E['operator'];
    /**
     * @return the value the expression evaluates to or `undefined` if the evaluation failed.
     */
    evaluate(expression: E, evaluationContext: EvaluationContext, wrapperFactories: WrapperFactoryProvider, strategy: EvaluationStrategy, validationContext: ValidationContext | undefined): InternalValueRepresentation | undefined;
}
export declare abstract class DefaultUnaryOperatorEvaluator<O extends InternalValueRepresentation, T extends InternalValueRepresentation> implements OperatorEvaluator<UnaryExpression> {
    readonly operator: UnaryExpressionOperator;
    private readonly operandValueTypeguard;
    constructor(operator: UnaryExpressionOperator, operandValueTypeguard: InternalValueRepresentationTypeguard<O>);
    protected abstract doEvaluate(operandValue: O, expression: UnaryExpression, context: ValidationContext | undefined): T | undefined;
    evaluate(expression: UnaryExpression, evaluationContext: EvaluationContext, wrapperFactories: WrapperFactoryProvider, strategy: EvaluationStrategy, validationContext: ValidationContext | undefined): T | undefined;
}
export declare abstract class DefaultBinaryOperatorEvaluator<L extends InternalValueRepresentation, R extends InternalValueRepresentation, T extends InternalValueRepresentation> implements OperatorEvaluator<BinaryExpression> {
    readonly operator: BinaryExpressionOperator;
    private readonly leftValueTypeguard;
    private readonly rightValueTypeguard;
    constructor(operator: BinaryExpressionOperator, leftValueTypeguard: InternalValueRepresentationTypeguard<L>, rightValueTypeguard: InternalValueRepresentationTypeguard<R>);
    protected abstract doEvaluate(leftValue: L, rightValue: R, expression: BinaryExpression, context: ValidationContext | undefined): T | undefined;
    evaluate(expression: BinaryExpression, evaluationContext: EvaluationContext, wrapperFactories: WrapperFactoryProvider, strategy: EvaluationStrategy, validationContext: ValidationContext | undefined): T | undefined;
}
/**
 * This class serves as a base for boolean operators that support short-circuit evaluation.
 * Short-circuit evaluation means that the right operand is not evaluated
 * if the resulting value can be determined by solely evaluating the left operand.
 */
export declare abstract class BooleanShortCircuitOperatorEvaluator implements OperatorEvaluator<BinaryExpression> {
    readonly operator: 'and' | 'or';
    constructor(operator: 'and' | 'or');
    protected abstract canSkipRightOperandEvaluation(leftValue: boolean): boolean;
    protected abstract getShortCircuitValue(): boolean;
    protected abstract doEvaluate(leftValue: boolean, rightValue: boolean): boolean;
    evaluate(expression: BinaryExpression, evaluationContext: EvaluationContext, wrapperFactories: WrapperFactoryProvider, strategy: EvaluationStrategy, validationContext: ValidationContext | undefined): boolean | undefined;
}
export declare abstract class DefaultTernaryOperatorEvaluator<FirstValue extends InternalValueRepresentation, SecondValue extends InternalValueRepresentation, ThirdValue extends InternalValueRepresentation, ReturnValue extends InternalValueRepresentation> implements OperatorEvaluator<TernaryExpression> {
    readonly operator: TernaryExpressionOperator;
    private readonly firstValueTypeguard;
    private readonly secondValueTypeguard;
    private readonly thirdValueTypeguard;
    constructor(operator: TernaryExpressionOperator, firstValueTypeguard: InternalValueRepresentationTypeguard<FirstValue>, secondValueTypeguard: InternalValueRepresentationTypeguard<SecondValue>, thirdValueTypeguard: InternalValueRepresentationTypeguard<ThirdValue>);
    protected abstract doEvaluate(firstValue: FirstValue, secondValue: SecondValue, thirdValue: ThirdValue, expression: TernaryExpression, context: ValidationContext | undefined): ReturnValue | undefined;
    evaluate(expression: TernaryExpression, evaluationContext: EvaluationContext, wrapperFactories: WrapperFactoryProvider, strategy: EvaluationStrategy, validationContext: ValidationContext | undefined): ReturnValue | undefined;
}
